/*
 * http://cc.byexamples.com/2007/01/20/print-color-string-without-ncurses/
 * Text attributes
 *        0    All attributes off
 *        1    Bold on
 *        4    Underscore (on monochrome display adapter only)
 *        5    Blink on
 *        7    Reverse video on
 *        8    Concealed on
 * 
 * Foreground colors
 *        30    Black
 *        31    Red
 *        32    Green
 *        33    Yellow
 *        34    Blue
 *        35    Magenta
 *        36    Cyan
 *        37    White
 * 
 * Background colors
 *        40    Black
 *        41    Red
 *        42    Green
 *        43    Yellow
 *        44    Blue
 *        45    Magenta
 *        46    Cyan
 *        47    White
 */
#include "defs.h"

#include <stdio.h>
#include <string.h>

int html;

int months_days[12] = {
  31, DAYS_FEBRUARY, 31, 30, 31, 30,
  31, 31, 30, 31, 30, 31
};

int ferie[12][32];
int cp[12][32];
int rtt[12][32];
int my_cp[12][32];
int my_rtt[12][32];

int worked[12][32];

int day[12][32];
int work[12][32];

float hours[12][32];
float hours_per_day[7];

const char *days[] = { "Mo", "Tu", "We", "Th", "Fr", "Sa", "Su" };
const char *months[] = {
  "January", "February", "March", "April", "May", "June",
  "July", "August", "September", "October", "November", "December"
};

int worked_days_in_month(int i)
{
  int j;
  int n = 0;
  for (j = 1; j <= months_days[i]; j++)
    if (work[i][j] == WORK || work[i][j] == WORKED) n++;
  return n;
}

void output_space(int d)
{
  int i;

  for (i = 0; i < d; i++) printf("   ");
}

void output_end_space(int d)
{
  int i;

  d = 6 - d;
  for (i = 0; i < d; i++) printf("   ");
}

void print_color(int w)
{
  switch (w) {
    case WORK:
      if (html) printf("<span>");
      break;
    case WE:
      if (html) printf("<span class=\"we\">");
      else printf("\x1b[1;46;30m");
      break;
    case FERIE:
      if (html) printf("<span class=\"ferie\">");
      else printf("\x1b[1;41;37m");
      break;
    case CP:
      if (html) printf("<span class=\"cp\">");
      else printf("\x1b[1;43;30m");
      break;
    case RTT:
      if (html) printf("<span class=\"rtt\">");
      else printf("\x1b[1;42;30m");
      break;
    case MY_CP:
      if (html) printf("<span class=\"my_cp\">");
      else printf("\x1b[1;44;37m");
      break;
    case MY_RTT:
      if (html) printf("<span class=\"my_rtt\">");
      else printf("\x1b[1;45;37m");
      break;
    case WORKED:
      if (html) printf("<span class=\"worked\">");
      else printf("\x1b[47;30m");
      break;
  }
}

void end_color(void)
{
  if (html) printf("</span>"); else printf("\x1b[0m");
}

void output_day(int d, int w)
{
  printf(" ");
  print_color(w);
  printf("%2d", d);
  end_color();
}

int main(int n, char **v)
{
  int i, j;
  int d;
  int nb_work = 0;
  int nb_rtt = 0;
  int nb_cp = 0;
  int nb_worked = 0;
  int nb_ferie = 0;
  int nb_we = 0;
  float nb_hours_todo = 0;
  float nb_hours_done = 0;

  for (i = 1; i < n; i++) {
    if (!strcmp(v[i], "-html")) html = 1;
    else {
      printf("usage: cal [options]\n");
      printf("options:\n");
      printf("    -html: output as HTML\n");
      return 0;
    }
  }

  DEF_FERIE;
  DEF_CP;
  DEF_RTT;

  DEF_MY_CP;
  DEF_MY_RTT;

  DEF_WORKED;

  /* feed the kind of day of each day */
  d = DAY_1;
  for (i = 0; i < 12; i++)
    for (j = 1; j <= months_days[i]; j++) {
      day[i][j] = d;
      d++;
      d %= 7;
    }

  DEF_HOURS_PER_DAY;

  /* hours per day */
  for (i = 0; i < 12; i++)
    for (j = 1; j <= months_days[i]; j++)
      hours[i][j] = hours_per_day[day[i][j]];

  /* hours per day: special days */
  DEF_HOURS;

  /* feed the work array */
  for (i = 0; i < 12; i++)
    for (j = 1; j <= months_days[i]; j++) {
      if (day[i][j] == SATURDAY || day[i][j] == SUNDAY) {
        /* week-end can be ferie, but nothing else */
        if (rtt[i][j] || cp[i][j] || my_rtt[i][j] || my_cp[i][j] ||
            worked[i][j]) {
          printf("bad definition of days (%s %d)\n", months[i], j);
          return 1;
        }
        work[i][j] = WE;
      }
      /* only one of ferie/cp/rtt/my_cp/my_rtt/worked is allowed
       * for each day
       */
      if (ferie[i][j] + cp[i][j] + rtt[i][j]
          + my_cp[i][j] + my_rtt[i][j] + worked[i][j] > 1) {
          printf("bad definition of day %s %d\n", months[i], j);
          return 1;
      }
      if (ferie[i][j]) work[i][j] = FERIE;
      if (cp[i][j]) work[i][j] = CP;
      if (rtt[i][j]) work[i][j] = RTT;
      if (my_cp[i][j]) work[i][j] = MY_CP;
      if (my_rtt[i][j]) work[i][j] = MY_RTT;
      if (worked[i][j]) work[i][j] = WORKED;

      if (work[i][j] == WORK) { nb_work++; nb_hours_todo += hours[i][j]; }
      if (work[i][j] == CP || work[i][j] == MY_CP) nb_cp++;
      if (work[i][j] == RTT || work[i][j] == MY_RTT) nb_rtt++;
      if (work[i][j] == WORKED) { nb_worked++; nb_hours_done += hours[i][j]; }
      if (work[i][j] == FERIE) nb_ferie++;
      if (work[i][j] == WE) nb_we++;
    }

  if (html) {
    printf("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\"\n");
    printf("    \"http://www.w3.org/TR/html4/strict.dtd\">\n");
    printf("<html>\n");
    printf("<head>\n");
    printf("<title>Calendar %d</title>\n", YEAR);
    printf(
  "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n");
    printf("<style type=\"text/css\">\n");
    printf("<!--\n");
    printf("body { font-size: 7pt; }\n");
    printf("span.we { background: lightblue; }\n");
    printf("span.ferie { background: red; color:white; }\n");
    printf("span.cp { background: orange; }\n");
    printf("span.rtt { background: green; }\n");
    printf("span.my_cp { background: blue; color: white; }\n");
    printf("span.my_rtt { background: magenta; color: white; }\n");
    printf("span.worked { background: black; color: white; }\n");
    printf("-->\n");
    printf("</style>\n");
    printf("</head>\n");
    printf("<body>\n\n");
    printf("<h1>Calendar %d</h1>\n\n", YEAR);
    printf("<p>\n<pre>\n");
  }

  for (i = 0; i < 12; i++) {
    int n = worked_days_in_month(i);
    if (i == 0) output_space(day[i][1]);
    for (j = 1; j <= months_days[i]; j++) {
      output_day(j, work[i][j]);
      if (day[i][j] == SUNDAY && j <= 7) printf("  %s (%d)\n", months[i], n);
      else if (day[i][j] == SUNDAY /*&& j != months_days[i] */) printf("\n");
    }
#if 0
    output_end_space(day[i][j-1]);
    printf("  %s (%d)\n\n", months[i], n);
#endif
  }
  printf("\n");
  printf("\n                       Total (%d)\n", nb_work + nb_worked);
  printf("Done: %d (%g%%) TODO: %d\n", nb_worked,
         (double)100*nb_worked/(nb_work+nb_worked), nb_work);
  printf("CP: %d RTT: %d, FERIE: %d, WE: %d\n",
         nb_cp, nb_rtt, nb_ferie, nb_we);
  print_color(WE); printf("WE"); end_color(); printf(" ");
  print_color(FERIE); printf("FERIE"); end_color(); printf(" ");
  print_color(CP); printf("CP"); end_color(); printf(" ");
  print_color(RTT); printf("RTT"); end_color(); printf(" ");
  print_color(MY_CP); printf("MY_CP"); end_color(); printf(" ");
  print_color(MY_RTT); printf("MY_RTT"); end_color(); printf("\n");
  printf("hours done: %g todo: %g (total: %g)\n",
         nb_hours_done, nb_hours_todo, nb_hours_done + nb_hours_todo);

  if (html) printf("</pre>\n\n</body>\n</html>\n");

  return 0;
}
